package org.kantega.jexmec.logging;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.PluginManagerListener;
import org.kantega.jexmec.events.PluginClassLoaderEvent;
import org.kantega.jexmec.events.PluginLoadingExceptionEvent;
import org.kantega.jexmec.events.PluginManagerEvent;
import org.kantega.jexmec.events.PluginRegistrationEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * A plugin manager listener logging all events to an SLF4J logger
 */
public class LoggingPluginManagerListener<P> extends PluginManagerListener<P> {
    private final Logger logger;

    /**
     * Creates a LoggingPluginManagerListener logging to the given Logger.
     * @param logger the Logger to log events to
     */
    public LoggingPluginManagerListener(Logger logger) {
        this.logger = logger;
    }

    /**
     * Creates a LoggingPluginManagerListener logging to the default Logger,
     * retrieved by calling <code>LoggerFactory.getLogger(LoggingPluginManagerListener.class)</code>
     */
    public LoggingPluginManagerListener() {
        this(LoggerFactory.getLogger(LoggingPluginManagerListener.class));
    }

    @Override
    public void beforePluginManagerStarted(PluginManagerEvent<P> event) {
        logger.info("Plugin manager is about to start: " + event.getPluginManager());
    }

    @Override
    public void afterPluginManagerStarted(PluginManagerEvent<P> event) {
        logger.info("Plugin manager has started: "  + event.getPluginManager());
    }

    @Override
    public void beforePluginManagerStopped(PluginManagerEvent<P> event) {
        logger.info("Plugin manager is about to stop: " + event.getPluginManager());
    }

    @Override
    public void afterPluginManagerStopped(PluginManagerEvent<P> event) {
        logger.info("Plugin manager has stopped: " + event.getPluginManager());
    }

    @Override
    public void beforeClassLoaderAdded(PluginClassLoaderEvent<P> event) {
        logger.info("Class loader is about to be added: " + event.getClassLoader()+" for provider " +event.getClassLoader());
    }

    @Override
    public void afterClassLoaderAdded(PluginClassLoaderEvent<P> event) {
        logger.info("Class loader has been added: " + event.getClassLoader() +" for provider " +event.getProvider());
    }

    @Override
    public void beforeClassLoaderRemoved(PluginClassLoaderEvent<P> event) {
        logger.info("Class loader is about to be removed: " + event.getClassLoader() +" for provider " +event.getClassLoader());
    }

    @Override
    public void afterClassLoaderRemoved(PluginClassLoaderEvent<P> event) {
        logger.info("Class loader has been removed: " + event.getClassLoader() +" for provider " +event.getProvider());
    }

    @Override
    public void beforeActivation(PluginRegistrationEvent<P> event) {
        logger.info("Plugins loaded from class loader " + event.getClassLoader() +" are about to be activated: " + event.getPlugins() +" for provider " +event.getProvider());
    }

    @Override
    public void afterActivation(PluginRegistrationEvent<P> event) {
        logger.info("Plugins loaded from class loader " + event.getClassLoader() + " have been activated: " + event.getPlugins()+" for provider " +event.getProvider());
    }

    @Override
    public void beforePassivation(PluginRegistrationEvent<P> event) {
        logger.info("Plugins loaded from class loader " + event.getClassLoader()+ " are about to be passivated: " + event.getPlugins()+" for provider " +event.getProvider());
    }

    @Override
    public void afterPassivation(PluginRegistrationEvent<P> event) {
        logger.info("Plugins loaded from class loader " + event.getClassLoader()+ " have been passivated: " + event.getPlugins() +" for provider " +event.getProvider());
    }

    @Override
    public void pluginsUpdated(List<P> plugins) {
        logger.info("Plugins list updated to: " + plugins);
    }

    @Override
    public void pluginLoadingFailedWithException(PluginLoadingExceptionEvent<P> event) {
        logger.error("Plugin loader " + event.getPluginLoader() + " threw " + event.getThrowable().getClass().getSimpleName() +" loading plugins from class loader " + event.getClassLoader(), event.getThrowable());
    }
}
